@@ -7,11 +7,15 @@ import os
 
 from jupyterlab_launcher import add_handlers, LabConfig
 from notebook.utils import url_path_join as ujoin
+from notebook.base.handlers import FileFindHandler
+from tornado.ioloop import IOLoop
 
 from .commands import (
-    get_app_dir, list_extensions, should_build, get_user_settings_dir
+    get_app_dir, list_extensions, should_build, get_user_settings_dir, watch,
+    build
 )
-from .settings_handler import settings_path, SettingsHandler
+
+from .build_handler import build_path, Builder, BuildHandler
 from ._version import __version__
 
 #-----------------------------------------------------------------------------
@@ -48,6 +52,7 @@ def load_jupyter_server_extension(nbapp):
     web_app = nbapp.web_app
     config = LabConfig()
 
+    config.name = 'JupyterLab'
     config.assets_dir = os.path.join(app_dir, 'static')
     config.settings_dir = os.path.join(app_dir, 'settings')
     config.page_title = 'JupyterLab Alpha Preview'
@@ -59,27 +64,23 @@ def load_jupyter_server_extension(nbapp):
     if hasattr(nbapp, 'core_mode'):
         core_mode = nbapp.core_mode
 
+    # Check for watch.
+    watch_mode = False
+    if hasattr(nbapp, 'watch'):
+        watch_mode = nbapp.watch
+
     # Check for an app dir that is local.
     if app_dir == here or app_dir == os.path.join(here, 'build'):
         core_mode = True
         config.settings_dir = ''
 
-    # Run core mode if explicit or there is no static dir and no
-    # installed extensions.
-    installed = list_extensions(app_dir)
-    fallback = not installed and not os.path.exists(config.assets_dir)
-
     web_app.settings.setdefault('page_config_data', dict())
+    web_app.settings['page_config_data']['buildAvailable'] = True
     web_app.settings['page_config_data']['token'] = nbapp.token
 
-    if not core_mode:
-        build_needed, msg = should_build(app_dir)
-        if build_needed:
-            nbapp.log.warn('Build required: %s' % msg)
-            web_app.settings['page_config_data']['buildRequired'] = msg
-
-    if core_mode or fallback:
+    if core_mode:
         config.assets_dir = os.path.join(here, 'build')
+        config.version = __version__
         if not os.path.exists(config.assets_dir):
             msg = 'Static assets not built, please see CONTRIBUTING.md'
             nbapp.log.error(msg)
@@ -89,23 +90,32 @@ def load_jupyter_server_extension(nbapp):
 
     if config.dev_mode:
         nbapp.log.info(DEV_NOTE_NPM)
-    elif core_mode or fallback:
+    elif core_mode:
         nbapp.log.info(CORE_NOTE.strip())
 
+    if core_mode:
+        schemas_dir = os.path.join(here, 'schemas')
+        config.themes_dir = os.path.join(here, 'themes')
+    else:
+        schemas_dir = os.path.join(app_dir, 'schemas')
+        config.themes_dir = os.path.join(app_dir, 'themes')
+
+    config.schemas_dir = schemas_dir
+    config.user_settings_dir = get_user_settings_dir()
+
+    if watch_mode:
+        if core_mode:
+            watch(here, nbapp.log)
+        else:
+            config.assets_dir = os.path.join(app_dir, 'staging', 'build')
+            build(app_dir=app_dir, logger=nbapp.log)
+            watch(os.path.join(app_dir, 'staging'), nbapp.log)
+
     add_handlers(web_app, config)
 
-    user_settings_dir = get_user_settings_dir()
-
-    if core_mode or fallback:
-        schemas_dir = os.path.join(here, 'schemas')
-    else:
-        schemas_dir = os.path.join(app_dir, 'schemas')
-
     base_url = web_app.settings['base_url']
-    settings_url = ujoin(base_url, settings_path)
-    settings_handler = (settings_url, SettingsHandler, {
-        'schemas_dir': schemas_dir,
-        'settings_dir': user_settings_dir
-    })
-    nbapp.log.error('shemas_dir: %s' % schemas_dir)
-    web_app.add_handlers(".*$", [settings_handler])
+    build_url = ujoin(base_url, build_path)
+    builder = Builder(nbapp.log, core_mode, app_dir)
+    build_handler = (build_url, BuildHandler, {'builder': builder})
+
+    web_app.add_handlers(".*$", [build_handler])
